"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadBlock = exports.loadBlocks = exports.loadDockerBlocks = exports.buildBlock = exports.watchBlock = exports.buildBlocksOrBundles = exports.cleanNodeFiles = exports.updateVersions = void 0;
const backlib_1 = require("backlib");
const fs = __importStar(require("fs-extra"));
const Path = __importStar(require("path"));
const builder_1 = require("./builder");
const utils_1 = require("./utils");
const vdev_config_1 = require("./vdev-config");
// --------- /Public Types --------- //
async function updateVersions(config) {
    var _a, _b;
    if (!config) {
        config = await (0, vdev_config_1.loadVdevConfig)();
    }
    const versionFiles = (_b = (_a = config.version) === null || _a === void 0 ? void 0 : _a.files) !== null && _b !== void 0 ? _b : null;
    // if we do not have version files, we skip.
    if (versionFiles == null) {
        return;
    }
    const newVersion = config.__version__;
    let firstUpdate = false; // flag that will be set 
    try {
        for (let file of versionFiles) {
            const originalContent = (await fs.readFile(file, 'utf8')).toString();
            const isHTML = file.toLowerCase().endsWith('html');
            let fileAppVersion = getVersion(originalContent, isHTML);
            if (newVersion !== fileAppVersion) {
                // On the first update needed, we start the log section for the version update
                if (!firstUpdate) {
                    console.log(`----- Version Update: Updating '__version__ = ".."' or '?v=..' to ${newVersion} `);
                    firstUpdate = true;
                }
                console.log(`Changing  ${fileAppVersion} -> ${newVersion} in file: ${file}`);
                let newContent = replaceVersion(originalContent, newVersion, isHTML);
                await fs.writeFile(file, newContent, 'utf8');
            }
            else {
                // Note: for now, we do not log when nothing to do.
            }
        }
    }
    catch (ex) {
        throw new Error(`ERROR while doing versioning files - ${ex.message}`);
    }
    // if we have at least one update, we close the log section.
    if (firstUpdate) {
        console.log('----- /Version Update: done');
    }
}
exports.updateVersions = updateVersions;
// FIXME: Needs to look at the "blocks" from the config
async function cleanNodeFiles() {
    const filesToDelete = ['./package-lock.json', './node_blockules'];
    const blocks = await loadBlocks();
    // dirs.unshift('./'); // we do not clean the base dir, as it is easy to do by hand, and then, scripts won't work
    // TODO: probably need to add web-server as well
    for (const block of Object.values(blocks)) {
        const dir = block.dir;
        for (let fName of filesToDelete) {
            const fileToDelete = Path.join(dir, fName);
            if ((await fs.pathExists(fileToDelete))) {
                await (0, backlib_1.saferRemove)(fileToDelete);
            }
        }
    }
}
exports.cleanNodeFiles = cleanNodeFiles;
/**
 *
 * @param names List of block name or block/bundle names
 */
async function buildBlocksOrBundles(names) {
    if (names.length === 0) {
        const blocks = await loadBlocks();
        for (let block of Object.values(blocks)) {
            await buildBlock(block);
        }
    }
    else {
        for (let name of names) {
            const blockAndBundle = name.split('/');
            // if only .length === 1 then blockAndBundle[1] which is fine
            await buildBlock(blockAndBundle[0], { onlyBundleName: blockAndBundle[1] });
        }
    }
}
exports.buildBlocksOrBundles = buildBlocksOrBundles;
// TODO: needs to add support for any block rule watch
// TODO: needs to add support for only one bundle watch
async function watchBlock(blockName) {
    const blocks = await loadBlocks();
    const block = blocks[blockName];
    await buildBlock(block, { watch: true });
}
exports.watchBlock = watchBlock;
const blockBuilt = new Set();
/**
 * Build a and block and eventually a bundle
 * @param blockName
 * @param opts
 */
async function buildBlock(blockOrName, opts) {
    let block;
    //// get the bloxc
    if (typeof blockOrName == 'string') {
        const blockByNames = await loadBlocks();
        block = blockByNames[blockOrName];
        if (!block) {
            throw new Error(`No block found for blockeName ${blockOrName}.`);
        }
    }
    else {
        block = blockOrName;
    }
    //// if already built, skip
    if (blockBuilt.has(block.name)) {
        console.log(`-------- Block ${block.name} already built - SKIP\n`);
        return;
    }
    //// execute the build
    console.log(`-------- Building Block ${block.name} ${block.dir}`);
    const builders = await (0, builder_1.getBuilders)(block);
    const startBlock = (0, utils_1.now)();
    for (const builder of builders) {
        const startBuilder = (0, utils_1.now)();
        console.log(`---- running builder ${builder.name}`);
        await builder.build(block, opts === null || opts === void 0 ? void 0 : opts.watch);
        await (0, utils_1.printLog)(`---- running builder ${builder.name} DONE`, startBuilder);
        console.log();
    }
    await (0, utils_1.printLog)(`-------- Building Block ${block.name} DONE`, startBlock);
    console.log();
    blockBuilt.add(block.name);
}
exports.buildBlock = buildBlock;
// --------- Public Loaders --------- //
async function loadDockerBlocks() {
    const blocks = await loadBlocks();
    const dockerBlocks = {};
    for (let block of Object.values(blocks)) {
        const hasDockerfile = await fs.pathExists(Path.join(block.dir, 'Dockerfile'));
        if (hasDockerfile) {
            dockerBlocks[block.name] = block;
        }
    }
    return dockerBlocks;
}
exports.loadDockerBlocks = loadDockerBlocks;
async function loadBlocks() {
    const rawConfig = await (0, vdev_config_1.loadVdevConfig)();
    const rawBlocks = rawConfig.blocks;
    const base = {
        system: rawConfig.system,
        __version__: rawConfig.__version__,
        imageTag: rawConfig.imageTag
    };
    // build the services map from the raw services list
    const blockByName = rawBlocks.map((item) => {
        let block;
        let name;
        // initialize the object
        if (typeof item === 'string') {
            block = {
                name: item
            };
        }
        else {
            if (!item.name) {
                throw new Error(`the build config file vdev.yaml has a block without '.name'`);
            }
            // we do a shallow clone			
            block = { ...item };
        }
        //// add the system/version (they should not been set in the block anyway)
        if (block.system != null || block.__version__ != null) {
            throw new Error(`.system or .__version__ cannot be set at the block level (but found in ${block.name})`);
        }
        Object.assign(block, base);
        // if the block does not have a dir, then, build it with the parent one
        if (!block.dir) {
            block.dir = Path.join(rawConfig.baseBlockDir, `${block.name}/`);
        }
        return block;
    }).reduce((map, block) => {
        map[block.name] = block;
        return map;
    }, {});
    return blockByName;
}
exports.loadBlocks = loadBlocks;
async function loadBlock(name) {
    const blocks = await loadBlocks();
    const block = blocks[name];
    if (!block) {
        throw new Error(`Block ${name} not found`);
    }
    return block;
}
exports.loadBlock = loadBlock;
// --------- /Public Loaders --------- //
//#region    ---------- version Utils ---------- 
/** Return the first version found. For html, looks for the `src|href=....?v=___` and for other files the version = ... */
function getVersion(content, isHtml = false) {
    // look for the href or src ?v=...
    if (isHtml) {
        const rgx = /<.*(href|src).*?v=(.*?)(\"|\&)/gm;
        const r = rgx.exec(content);
        if (r != null && r.length > 2) {
            return r[2];
        }
        else {
            return null;
        }
    }
    // look for the version = ...
    else {
        var rx = new RegExp('__version__' + '\\s*[=:]\\s*[\'"](.*)[\'"]', 'i');
        var match = content.match(rx);
        return (match) ? match[1] : null;
    }
}
function replaceVersion(content, value, isHtml = false) {
    if (isHtml) {
        const rgxRep = /(<.*(href|src).*?v=).*?(\"|\&.*)/g;
        // $2 not is not used because it is included as part of $1
        return content.replace(rgxRep, '$1' + value + '$3');
    }
    else {
        var rx = new RegExp('(.*' + '__version__' + '\\s*[=:]\\s*["\']).*([\'"].*)', 'i');
        content = content.replace(rx, '$1' + value + '$2');
        return content;
    }
}
//#endregion ---------- /version Utils ---------- 
// --------- /Private Utils --------- //
