"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const backlib_1 = require("backlib");
const fs_extra_1 = require("fs-extra");
const p_spawn_1 = require("p-spawn");
const Path = __importStar(require("path"));
const builder_1 = require("./builder");
const utils_1 = require("./utils");
//#region    ---------- builder - npm install - 'package.json' ---------- 
(0, builder_1.registerBuilder)({
    name: 'npm_install',
    predicate: 'package.json',
    order: -1024,
    build: async (block) => {
        await (0, p_spawn_1.spawn)('npm', ['install'], { cwd: block.dir });
    }
});
//#endregion ---------- /builder - npm install - 'package.json' ---------- 
//#region    ---------- builder - tsc - 'tsconfig.json' ---------- 
(0, builder_1.registerBuilder)({
    name: 'tsc',
    predicate: 'tsconfig.json',
    order: 0,
    build: async (block, watch = false) => {
        const distDirNeedsDelete = false;
        const distDir = Path.join(block.dir, '/dist/');
        const distDirExist = await (0, fs_extra_1.pathExists)(distDir);
        // if we have distDirExist, check that it define as compileOptions.outDir in 
        if (distDirExist) {
            const tsconfigObj = await (0, utils_1.readJsonFileWithComments)(Path.join(block.dir, 'tsconfig.json'));
            let outDir = tsconfigObj.compilerOptions.outDir;
            outDir = (outDir) ? Path.join(block.dir, outDir, '/') : null; // add a ending '/' to normalize all of the dir path with ending / (join will remove duplicate)
            if (outDir === distDir) {
                console.log(`tsc prep - deleting tsc distDir ${distDir}`);
                await (0, backlib_1.saferRemove)(distDir);
            }
            else {
                console.log(`tss prep - skipping tsc distDir ${distDir} because does not match tsconfig.json compilerOptions.outDir ${outDir}`);
            }
        }
        // Assume there is a typescript installed in the root project
        const args = ['-p', block.dir];
        await (0, p_spawn_1.spawn)('./node_modules/.bin/tsc', args);
        //// Enhancement, support watch for tsc vdev watch... project
        ////  	- cons: will need to know what to call on update (for example, restart.sh is app specific)
        ////    - prop: Perhaps, have this part of a hook on watch, a notify (would need a vdev.js or some sort)
        // if (watch){
        // 	const watchTscPromise = spawn('./node_modules/.bin/tsc', [...args, '-w']);
        // } else {
        // 	await spawn('./node_modules/.bin/tsc', args);
        // }
    }
});
//#endregion ---------- /builder - tsc - 'tsconfig.json' ---------- 
//#region    ---------- builder - rollup - 'rollup.config.js' ---------- 
(0, builder_1.registerBuilder)({
    name: 'rollup',
    predicate: 'rollup.config.js',
    order: 0,
    replace: ['tsc'],
    build: async (block, watch = false) => {
        // Assume there is a typescript installed in the root project
        const nodeModuleDir = Path.relative(block.dir, './node_modules/');
        const bin = Path.join(nodeModuleDir, '.bin/rollup');
        const args = ['-c'];
        if (watch)
            args.push('-w');
        const pproc = (0, p_spawn_1.spawn)(bin, args, { cwd: block.dir });
        if (!watch)
            await pproc;
    }
});
//#endregion ---------- /builder - rollup - 'rollup.config.js' ----------
//#region    ---------- builder - pcss - 'pcss.config.js' ---------- 
(0, builder_1.registerBuilder)({
    name: 'pcss',
    predicate: 'pcss.config.js',
    order: 0,
    build: async (block, watch = false) => {
        // Assume there is a typescript installed in the root project
        const nodeModuleDir = Path.relative(block.dir, './node_modules/');
        const bin = Path.join(nodeModuleDir, '.bin/pcss');
        const args = [];
        if (watch)
            args.push('-w');
        const pproc = (0, p_spawn_1.spawn)(bin, args, { cwd: block.dir });
        if (!watch)
            await pproc;
    }
});
//#endregion ---------- /builder - pcss - 'pcss.config.js' ----------
//#region    ---------- builder - hbs - 'hbs.config.js' ---------- 
(0, builder_1.registerBuilder)({
    name: 'hbs',
    predicate: 'hbs.config.js',
    order: 0,
    build: async (block, watch = false) => {
        // Assume there is a typescript installed in the root project
        const nodeModuleDir = Path.relative(block.dir, './node_modules/');
        const bin = Path.join(nodeModuleDir, '.bin/hbs');
        const args = [];
        if (watch)
            args.push('-w');
        const pproc = (0, p_spawn_1.spawn)(bin, args, { cwd: block.dir });
        if (!watch)
            await pproc;
    }
});
//#endregion ---------- /builder - hbs - 'hbs.config.js' ----------
