"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.specialPathResolve = void 0;
const backlib_1 = require("backlib");
const chokidar = __importStar(require("chokidar"));
const fs_extra_1 = require("fs-extra");
const Path = __importStar(require("path"));
const utils_min_1 = require("utils-min");
const builder_1 = require("./builder");
const builder_web_bundlers_processors_1 = require("./builder-web-bundlers-processors");
const utils_1 = require("./utils");
const now = Date.now;
const bundle_types = ['ts', 'js', 'pcss', 'tmpl', 'html'];
async function buildBundles(block, type, watch) {
    var _a;
    const typeSet = new Set((0, utils_min_1.asArray)(type));
    for (const bundle of block.webBundles) {
        if (typeSet.has(bundle.type)) {
            const type = bundle.type;
            const bundler = bundlers[type];
            await ensureDist(bundle);
            let start = now();
            // if js or ts, watch is part of the bundler
            if (bundle.type === 'js' || bundle.type === 'ts') {
                if (bundle.watch) {
                    console.log(`WARNING - Ignoring '.watch' property for bundle ${bundle.name} for .js/.ts processing
				as rollup watches dependencies. (advice: remove this .watch property for clarity)`);
                }
                await bundler(block, bundle, watch);
            }
            // otherwise, implement the watch
            else {
                await bundler(block, bundle);
                if (watch) {
                    const toWatch = (_a = bundle.watch) !== null && _a !== void 0 ? _a : bundle.entries;
                    let watcher = chokidar.watch(toWatch, { persistent: true });
                    // TODO: Needs to use a call reducer
                    watcher.on('change', async function (filePath, stats) {
                        if (filePath.endsWith(`.${bundle.type}`)) {
                            await bundler(block, bundle);
                        }
                    });
                }
            }
            if (watch) {
                await (0, utils_1.printLog)(`Starting watch mode for ${block.name}/${bundle.name} (${bundle.dist})`, start);
            }
            else {
                await (0, utils_1.printLog)(`Building bundle ${block.name}/${bundle.name}`, start, bundle.dist);
            }
        }
    }
}
async function hasBundleType(block, ...type) {
    if (block.webBundles) {
        const typeSet = new Set(type);
        for (let bundle of block.webBundles) {
            await initWebBundle(block, bundle);
            if (typeSet.has(bundle.type))
                return true;
        }
    }
    return false;
}
//#region    ---------- builder - ts_bundler ---------- 
(0, builder_1.registerBuilder)({
    name: 'ts_bundler',
    predicate: async (block) => {
        return hasBundleType(block, 'ts');
    },
    order: 0,
    replace: ['tsc'],
    build: async (block, watch = false) => {
        await buildBundles(block, 'ts', watch);
    }
});
//#endregion ---------- /builder - ts_bundler ----------
//#region    ---------- builder - js_bundler ---------- 
(0, builder_1.registerBuilder)({
    name: 'js_bundler',
    predicate: async (block) => {
        return hasBundleType(block, 'js');
    },
    order: 0,
    build: async (block, watch = false) => {
        await buildBundles(block, 'js', watch);
    }
});
//#endregion ---------- /builder - js_bundler ----------
//#region    ---------- builder - pcss_bundler ---------- 
(0, builder_1.registerBuilder)({
    name: 'pcss_bundler',
    predicate: async (block) => {
        return hasBundleType(block, 'pcss');
    },
    order: 0,
    build: async (block, watch = false) => {
        await buildBundles(block, 'pcss', watch);
    }
});
//#endregion ---------- /builder - pcss_bundler ----------
//#region    ---------- builder - tmpl_bundler ---------- 
(0, builder_1.registerBuilder)({
    name: 'tmpl_bundler',
    predicate: async (block) => {
        return hasBundleType(block, 'tmpl', 'html');
    },
    order: 0,
    build: async (block, watch = false) => {
        await buildBundles(block, ['tmpl', 'html'], watch);
    }
});
//#endregion ---------- /builder - tmpl_bundler ----------
//#region    ---------- webBundler utils ---------- 
const rollupOptionsDefaults = {
    ts: {
        watch: false,
        ts: true,
        tsconfig: './tsconfig.json'
    },
    js: {
        watch: false,
        ts: false
    }
};
// bundlers by type (which is file extension without '.')
const bundlers = {
    ts: buildTsBundler,
    pcss: buildPcssBundler,
    tmpl: buildTmplBundler,
    html: buildTmplBundler,
    js: buildJsBundler
};
/**
 * Initialize all of the bundle properties accordingly.
 * This allow the bundlers and other logic to not have to worry about default values and path resolution.
 */
async function initWebBundle(block, bundle) {
    if (bundle.type != null)
        return; // assume already initalized
    bundle.type = Path.extname((0, utils_1.asNames)(bundle.entries)[0]).substring(1);
    // for now, just take the block.dir
    bundle.dir = specialPathResolve('', block.dir, bundle.dir);
    // Make the entries relative to the Block
    bundle.entries = (0, utils_1.asNames)(bundle.entries).map((f) => specialPathResolve('', bundle.dir, f));
    // If bundle.watch, same as entries above
    if (bundle.watch) {
        // Make the watch relative to the Block
        bundle.watch = (0, utils_1.asNames)(bundle.watch).map((f) => specialPathResolve('', bundle.dir, f));
    }
    // resolve the dist
    bundle.dist = specialPathResolve('', block.baseDistDir, bundle.dist);
    // rollup initialization
    if (bundle.type === 'ts' || bundle.type === 'js') {
        // get the base default options for this type
        const rollupOptionsDefault = rollupOptionsDefaults[bundle.type];
        // override default it if bundle has one
        const rollupOptions = (bundle.rollupOptions) ? { ...rollupOptionsDefault, ...bundle.rollupOptions }
            : { ...rollupOptionsDefault };
        // resolve tsconfig
        if (rollupOptions.tsconfig) {
            rollupOptions.tsconfig = specialPathResolve('', bundle.dir, rollupOptions.tsconfig);
        }
        // set the new optoins back.
        bundle.rollupOptions = rollupOptions;
    }
}
async function ensureDist(bundle) {
    const distDir = Path.dirname(bundle.dist);
    await (0, fs_extra_1.ensureDir)(distDir);
}
/**
 * Special resolve that
 *   - if finalPath null, then, return dir.
 *   - resolve any path to dir if it starts with './' or '../',
 *   - absolute path if it starts with '/'
 *   - baseDir if the path does not start with either '/' or './'
 * @param baseDir
 * @param dir
 * @param finalPath dir or file path
 */
function specialPathResolve(baseDir, dir, finalPath) {
    if (finalPath == null) {
        return dir;
    }
    if (finalPath.startsWith('/')) {
        return finalPath;
    }
    if (finalPath.startsWith('./') || finalPath.startsWith('../')) {
        return Path.join(dir, finalPath);
    }
    return Path.join(baseDir, finalPath);
}
exports.specialPathResolve = specialPathResolve;
//#endregion ---------- /webBundler utils ----------
//#region    ---------- Private Bundlers ---------- 
async function buildTsBundler(block, bundle, watch) {
    // TODO: need to re-enable watch
    try {
        if (watch) {
            bundle.rollupOptions.watch = true;
        }
        // resolve all of the entries (with glob)
        const allEntries = await resolveGlobs(bundle.entries);
        await (0, builder_web_bundlers_processors_1.rollupFiles)(allEntries, bundle.dist, bundle.rollupOptions);
    }
    catch (ex) {
        // TODO: need to move exception ahndle to the caller
        console.log("BUILD ERROR - something when wrong on rollup\n\t", ex);
        console.log("Empty string was save to the app bundle");
        console.log("Trying saving again...");
        return;
    }
}
async function buildPcssBundler(block, bundle, watch) {
    const allEntries = await resolveGlobs(bundle.entries);
    await (0, builder_web_bundlers_processors_1.pcssFiles)(allEntries, bundle.dist);
}
async function buildTmplBundler(block, bundle, watch) {
    const allEntries = await resolveGlobs(bundle.entries);
    await (0, builder_web_bundlers_processors_1.tmplFiles)(allEntries, bundle.dist);
}
async function buildJsBundler(block, bundle, watch) {
    if (watch) {
        bundle.rollupOptions.watch = true;
    }
    const allEntries = await resolveGlobs(bundle.entries);
    await (0, builder_web_bundlers_processors_1.rollupFiles)(allEntries, bundle.dist, bundle.rollupOptions);
}
//#endregion ---------- /Private Bundlers ----------
/** Since 0.11.18 each string glob is sorted within their match, but if globs is an array, the order of each result glob result is preserved. */
async function resolveGlobs(globs) {
    if (typeof globs === 'string') {
        return (0, backlib_1.glob)(globs);
    }
    else {
        const lists = [];
        for (const globStr of globs) {
            const list = await (0, backlib_1.glob)(globStr);
            lists.push(list);
        }
        return lists.flat();
    }
}
